/**
 * CompanyServiceImpl.java
 *
 * Created on 28 Agu 2010 17:51:18
 */
package org.blueoxygen.cimande.company.service;

import java.util.List;

import org.blueoxygen.cimande.company.Company;
import org.blueoxygen.cimande.persistence.hibernate.dao.HibernatePersistenceDaoManager;
import org.hibernate.Criteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author Dian Aditya
 * 
 */
@Repository
@Transactional(readOnly = true)
public class CompanyServiceImpl extends HibernatePersistenceDaoManager<Company>
		implements CompanyService {

	@Transactional
	public void save(Company company) {
		if (company != null) {
			if (company.getId() == null || "".equalsIgnoreCase(company.getId())) {
				company.getLogInformation().setCreateBy(
						getCurrentUser().getId());
				company.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				company.getLogInformation().setCreateDate(getCurretTime());
				company.getLogInformation().setLastUpdateDate(getCurretTime());
				company.getLogInformation().setActiveFlag(1);

				persist(company);
			} else {
				company.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				company.getLogInformation().setLastUpdateDate(getCurretTime());

				merge(company);
			}
		}
	}

	@Transactional
	public void delete(Company company) {
		if (company != null && company.getId() != null) {
			company = getById(Company.class, company.getId());

			remove(company);
		}
	}

	@Transactional(readOnly = true)
	public Company getCompanyById(String id) {
		return getById(Company.class, id);
	}

	@Transactional(readOnly = true)
	public List<Company> getAllCompanies(String orderby) {
		return findAllSorted(Company.class, orderby);
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<Company> findCompanies(String name, String address,
			String state, String zipNumber, String telephone, String faximile,
			String website, String email, String orderby, int maxResults,
			int firstResult) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Company.class);

		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("address", address + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("state", state + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("zip_number", zipNumber + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("telephone", telephone + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("faximile", faximile + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("website", website + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("email", email + "", MatchMode.ANYWHERE));

		criteria.setMaxResults(maxResults);
		criteria.setFirstResult(firstResult);

		return criteria.list();
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<Company> findCompaniesByField(String field, String value) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Company.class);
		criteria.add(Restrictions.like(field, value));

		return criteria.list();
	}

	@Transactional(readOnly = true)
	public int getRowCountByCriteria(String name, String address, String state,
			String zipNumber, String telephone, String faximile,
			String website, String email) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Company.class);

		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("address", address + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("state", state + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("zip_number", zipNumber + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("telephone", telephone + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("faximile", faximile + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("website", website + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("email", email + "", MatchMode.ANYWHERE));

		criteria.setProjection(Projections.rowCount());

		return new Integer(criteria.uniqueResult() + "");
	}

}
