package org.blueoxygen.cimande.descriptors.service;

import java.util.List;

import org.blueoxygen.cimande.descriptors.Descriptor;
import org.blueoxygen.cimande.persistence.hibernate.dao.HibernatePersistenceDaoManager;
import org.hibernate.Criteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/*******************************
 * @author Praditya Eldorado aldo.donal@gmail.com
 *******************************/

@Repository
@Transactional(readOnly = true)
public class DescriptorServiceImpl extends
		HibernatePersistenceDaoManager<Descriptor> implements DescriptorService {

	@Transactional
	public void save(Descriptor des) {
		if (des != null) {
			if (des.getId() == null || "".equalsIgnoreCase(des.getId())) {
				des.getLogInformation().setCreateBy(getCurrentUser().getId());
				des.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				des.getLogInformation().setCreateDate(getCurretTime());
				des.getLogInformation().setLastUpdateDate(getCurretTime());
				des.getLogInformation().setActiveFlag(1);
				persist(des);
			} else {
				des.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				des.getLogInformation().setLastUpdateDate(getCurretTime());

				merge(des);
			}

		}

	}

	@Transactional
	public void delete(Descriptor des) {
		if (des != null && des.getId() != null) {
			des = getById(Descriptor.class, des.getId());

			remove(des);
		}
	}

	@Transactional(readOnly = true)
	public Descriptor getDescriptorById(String id) {
		return getById(Descriptor.class, id);
	}

	@Transactional(readOnly = true)
	public List<Descriptor> getAllDescriptors(String orderby) {
		return findAllSorted(Descriptor.class, orderby);
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<Descriptor> findDescriptors(String name, String description,
			String orderby, int maxResults, int firstResult) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Descriptor.class);

		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("description", description + "",
				MatchMode.ANYWHERE));
		criteria.addOrder(Order.asc(orderby));
		criteria.setMaxResults(maxResults);
		criteria.setFirstResult(firstResult);

		return criteria.list();
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<Descriptor> findDescriptorsByField(String field, String value) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Descriptor.class);
		criteria.add(Restrictions.like(field, value));

		return criteria.list();
	}

	@Transactional(readOnly = true)
	public int getRowCountByCriteria(String name, String description) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Descriptor.class);

		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("description", description + "",
				MatchMode.ANYWHERE));

		criteria.setProjection(Projections.rowCount());

		return new Integer(criteria.uniqueResult() + "");
	}

}
