package org.blueoxygen.cimande.jobdescription.service;

import java.util.List;

import org.blueoxygen.cimande.jobdescription.JobDesc;
import org.blueoxygen.cimande.persistence.hibernate.dao.HibernatePersistenceDaoManager;
import org.hibernate.Criteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/*******************************
 * @author Praditya Eldorado aldo.donal@gmail.com
 *******************************/

@Repository
@Transactional(readOnly = true)
public class JobdescriptionServisImpl extends
		HibernatePersistenceDaoManager<JobDesc> implements JobdescriptionServis {
	@Transactional
	public void save(JobDesc job) {
		if (job != null) {
			if (job.getId() == null || "".equalsIgnoreCase(job.getId())) {
				job.getLogInformation().setCreateBy(getCurrentUser().getId());
				job.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				job.getLogInformation().setCreateDate(getCurretTime());
				job.getLogInformation().setLastUpdateDate(getCurretTime());
				// job.getLogInformation().setActiveFlag(1);
				persist(job);
			} else {
				job.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				job.getLogInformation().setLastUpdateDate(getCurretTime());

				merge(job);
			}

		}

	}

	@Transactional
	public void delete(JobDesc job) {
		if (job != null && job.getId() != null) {
			job = getById(JobDesc.class, job.getId());

			remove(job);
		}
	}

	@Transactional(readOnly = true)
	public JobDesc getJobDescById(String id) {
		return getById(JobDesc.class, id);
	}

	@Transactional(readOnly = true)
	public List<JobDesc> getAllJobDescs(String orderby) {
		return findAllSorted(JobDesc.class, orderby);
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<JobDesc> findJobDescs(String name, String description,
			String orderby, int maxResults, int firstResult) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				JobDesc.class);

		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("description", description + "",
				MatchMode.ANYWHERE));
		criteria.setMaxResults(maxResults);
		criteria.setFirstResult(firstResult);
		criteria.addOrder(Order.asc(orderby));
		return criteria.list();
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<JobDesc> findJobDescsByField(String field, String value) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				JobDesc.class);
		criteria.add(Restrictions.like(field, value));

		return criteria.list();
	}

	@Transactional(readOnly = true)
	public int getRowCountByCriteria(String name, String description) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				JobDesc.class);

		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("description", description + "",
				MatchMode.ANYWHERE));

		criteria.setProjection(Projections.rowCount());

		return new Integer(criteria.uniqueResult() + "");
	}

}
