package org.blueoxygen.cimande.security.user.service;

import java.util.List;

import org.blueoxygen.cimande.persistence.hibernate.dao.HibernatePersistenceDaoManager;
import org.blueoxygen.cimande.security.Name;
import org.blueoxygen.cimande.security.User;
import org.hibernate.Criteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

@Repository
@Transactional(readOnly = true)
public class UserServiceImpl extends HibernatePersistenceDaoManager<User>
		implements UserService {
	@Transactional
	public void save(User user) {
		if (user != null) {
			if (user.getId() == null || "".equalsIgnoreCase(user.getId())) {
				user.getLogInformation().setCreateBy(getCurrentUser().getId());
				user.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				user.getLogInformation().setCreateDate(getCurretTime());
				user.getLogInformation().setLastUpdateDate(getCurretTime());
				user.getLogInformation().setActiveFlag(1);
				persist(user);
			} else {
				user.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				user.getLogInformation().setLastUpdateDate(getCurretTime());
				merge(user);
			}
		}
	}

	@Transactional
	public void delete(User user) {
		if (user != null && user.getId() != null) {
			user = getById(User.class, user.getId());

			remove(user);
		}
	}

	@Transactional(readOnly = true)
	public User getUserbyId(String id) {
		return getById(User.class, id);
	}

	@Transactional(readOnly = true)
	public List<User> getAllUsers() {
		return findAll(User.class);
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<User> findUsers(String username, String name, String email,
			String address, String city, String orderby, int maxResults,
			int firstResult) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				User.class);
		criteria.add(Restrictions.like("username", username == null ? ""
				: username, MatchMode.ANYWHERE));
		System.out.println("UUU" + username + "");
		criteria.add(Restrictions.like("name.first", name == null ? "" : name,
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("email", email == null ? "" : email,
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("address.street1", address == null ? ""
				: address, MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("address.city",
				city == null ? "" : city, MatchMode.ANYWHERE));
		criteria.setMaxResults(maxResults);
		criteria.setFirstResult(firstResult);

		return criteria.list();
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<User> findUsersByField(String field, String value) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				User.class);
		criteria.add(Restrictions.like(field, value));

		return criteria.list();
	}

	@Transactional(readOnly = true)
	public int getRowCountByCriteria(String username, String name) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				User.class);

		criteria.add(Restrictions.like("username", username == null ? ""
				: username, MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("name.first", name + "",
				MatchMode.ANYWHERE));

		criteria.setProjection(Projections.rowCount());

		return new Integer(criteria.uniqueResult() + "");
	}

}
