/**
 * DropDownMenuGenerator.java
 *
 * Created on 24 Sep 2010 10:50:45
 */
package org.blueoxygen.cimande.sitemanager;

import java.util.List;

import org.apache.struts2.ServletActionContext;
import org.blueoxygen.cimande.commons.CimandeAction;
import org.blueoxygen.cimande.modulefunction.ModuleFunction;
import org.blueoxygen.cimande.persistence.hibernate.HibernateSessionFactory;
import org.blueoxygen.cimande.role.RolePrivilage;
import org.blueoxygen.cimande.role.RoleSite;
import org.blueoxygen.cimande.role.RoleSitePrivilage;
import org.blueoxygen.cimande.security.LoginFilter;
import org.blueoxygen.cimande.security.User;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * @author Dian Aditya
 *
 */
public class DropDownMenuGenerator extends CimandeAction {

	private @Autowired
	HibernateSessionFactory factory;

	private String roleId = "";
	private String siteId = "";

	private String script = "";
	private String jscript = "";

	public String getChilds(ModuleFunction function, Session session) {
		Criteria criteria = session.createCriteria(ModuleFunction.class);
		criteria.add(Restrictions.eq("moduleFunction.id", function.getId()));
		criteria.addOrder(Order.asc("description"));
		List<ModuleFunction> functions = criteria.list();

		String script = "";

		if (functions.size() > 0) {
			script += "<ul>";
			for (ModuleFunction mf : functions) {
				script += "<li><a href=\"#\">" + mf.getName() + "</a>";
				script += getChilds(mf, session);
				script += "</li>";
			}
			script += "</ul>";
		} else {
			String sActionFlag = "", sUrlAction = "";
			/*
			 * check the record in descriptor table if the action_flag = 1 means
			 * url_action else the node is url_descriptor value
			 */
			sActionFlag = String.valueOf(function.getModuleDescriptor()
					.getTypeFlag());

			if (sActionFlag.equals("1")) {
				sUrlAction = function.getModuleDescriptor().getUrlAction()
						+ "&";
			} else if (sActionFlag.equals("2")) {
				sUrlAction = request.getContextPath() + "/module/"
						+ function.getModuleDescriptor().getName() + "/";
			} else if (sActionFlag.equals("3")) {
				sUrlAction = request.getContextPath()
						+ "/module/window/genform?window.id="
						+ function.getModuleDescriptor().getWindow().getId()
						+ "&";
			} else {
				sUrlAction = request.getContextPath() + "/descriptor/"
						+ function.getModuleDescriptor().getUrlDescriptor()
						+ "?";
			}

			script += "<ul>";

			// new action <a href="output/hello.html">New</a>
			if (sActionFlag.equals("2")) {
				// - new_formula /{uri}/{new}
				script += "<li><a href=\"" + sUrlAction
						+ "create\" target=\"main\">New</a></li>";
			} else {
				script += "<li><a href=\"" + sUrlAction
						+ "action=new\" target=\"main\">New</a></li>";
			}

			// search action
			if (sActionFlag.equals("2")) {
				// - search_formula /{uri}/{new}
				script += "<li><a href=\"" + sUrlAction
						+ "filter\" target=\"main\">Search</a></li>";
			} else {
				script += "<li><a href=\"" + sUrlAction
						+ "action=search\" target=\"main\">Search</a></li>";
			}

			script += "</ul>";
		}

		return script;
	}

	public String execute() throws Exception {
		Session session = factory.createSession();
		String script = "";
		String jscript = "";

		User us = getCurrentUser();
		this.roleId = us.getRole().getId();
		this.siteId = (String) ServletActionContext.getRequest().getSession()
				.getAttribute(LoginFilter.LOGIN_CIMANDE_SITE);

		Criteria criteria = session.createCriteria(RoleSite.class);
		criteria.add(Restrictions.eq("role.id", roleId));
		criteria.add(Restrictions.eq("site.id", siteId));

		RoleSite roleSite = (RoleSite) criteria.uniqueResult();

		if (roleSite != null) {
			criteria = session.createCriteria(RoleSitePrivilage.class);
			criteria.add(Restrictions.eq("roleSite.id", roleSite.getId()));

			List<RoleSitePrivilage> privilages = criteria.list();

			for (int i = 0; i < privilages.size(); i++) {
				RoleSitePrivilage privilage = privilages.get(i);

				jscript += "j('#" + privilage.getId()
						+ "').menu({ flyOut: true, content: j('#"
						+ privilage.getModuleFunction().getId()
						+ "').html()});";

				script += "<a href=\"#"
						+ privilage.getModuleFunction().getId()
						+ "\" class=\"fg-button fg-button-icon-right ui-widget ui-state-default ui-corner-all\" id=\""
						+ privilage.getId()
						+ "\"><span class=\"ui-icon ui-icon-triangle-1-s\"></span>"
						+ privilage.getModuleFunction().getName() + "</a>";
				script += "<div id=\"" + privilage.getModuleFunction().getId()
						+ "\" class=\"hidden\">";

				script += getChilds(privilage.getModuleFunction(), session);

				script += "</div>";
			}

		} else {
			criteria = session.createCriteria(RolePrivilage.class);
			criteria.add(Restrictions.eq("role.id", roleId));

			List<RolePrivilage> privilages = criteria.list();

			for (RolePrivilage privilage : privilages) {
				jscript += "j('#" + privilage.getId()
						+ "').menu({ flyOut: true, content: j('#"
						+ privilage.getModuleFunction().getId()
						+ "').html()});";

				script += "<a href=\"#"
						+ privilage.getModuleFunction().getId()
						+ "\" class=\"fg-button fg-button-icon-right ui-widget ui-state-default ui-corner-all\" id=\""
						+ privilage.getId()
						+ "\"><span class=\"ui-icon ui-icon-triangle-1-s\"></span>"
						+ privilage.getModuleFunction().getName() + "</a>";
				script += "<div id=\"" + privilage.getModuleFunction().getId()
						+ "\" class=\"hidden\">";

				script += getChilds(privilage.getModuleFunction(), session);

				script += "</div>";
			}
		}

		this.script = script;
		this.jscript = jscript;

		return SUCCESS;
	}

	public String getScript() {
		return script;
	}

	public void setScript(String script) {
		this.script = script;
	}

	public String getJscript() {
		return jscript;
	}

	public void setJscript(String jscript) {
		this.jscript = jscript;
	}

}
