/**
 * 
 */
package org.blueoxygen.cimande.sitemanager.yuitree.treenode;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts2.ServletActionContext;
import org.blueoxygen.cimande.modulefunction.ModuleFunction;
import org.blueoxygen.cimande.role.RolePrivilage;
import org.blueoxygen.cimande.role.RoleSite;
import org.blueoxygen.cimande.role.RoleSitePrivilage;
import org.blueoxygen.cimande.security.LoginFilter;
import org.blueoxygen.cimande.security.User;
import org.blueoxygen.cimande.sitemanager.yuitree.TreeNode;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author Dian Aditya
 * 
 */
@Repository
@Transactional(readOnly = true)
public class TreeNodeFactoryImpl implements TreeNodeFactory {

	@Autowired
	private SessionFactory factory;

	public List<TreeNode> getTreeNodes(User user) {
		Session session = factory.getCurrentSession();
		List<TreeNode> treeNodes = new ArrayList<TreeNode>();

		String roleId = user.getRole().getId();
		String siteId = (String) ServletActionContext.getRequest().getSession()
				.getAttribute(LoginFilter.LOGIN_CIMANDE_SITE);

		Criteria criteria = session.createCriteria(RoleSite.class);
		criteria.add(Restrictions.eq("role.id", roleId));
		criteria.add(Restrictions.eq("site.id", siteId));

		RoleSite roleSite = (RoleSite) criteria.uniqueResult();

		if (roleSite != null) {
			Query query = session
					.createQuery("FROM "
							+ RoleSitePrivilage.class.getName()
							+ " s WHERE s.roleSite.id = :id ORDER BY s.moduleFunction.description");
			query.setParameter("id", roleSite.getId());

			List<RoleSitePrivilage> privilages = query.list();

			for (int i = 0; i < privilages.size(); i++) {
				RoleSitePrivilage privilage = privilages.get(i);
				TreeNode node = new TreeNode(privilage.getModuleFunction()
						.getId(), privilage.getModuleFunction()
						.getDescription(), false);
				treeNodes.add(node);
			}
		} else {
			Query query = session
					.createQuery("FROM "
							+ RolePrivilage.class.getName()
							+ " s WHERE s.role.id = :id ORDER BY s.moduleFunction.description");
			query.setParameter("id", roleSite.getId());

			List<RolePrivilage> privilages = query.list();

			for (RolePrivilage privilage : privilages) {
				TreeNode node = new TreeNode(privilage.getModuleFunction()
						.getId(), privilage.getModuleFunction()
						.getDescription(), false);
				treeNodes.add(node);
			}
		}

		return treeNodes;
	}

	public List<TreeNode> getTreeChild(String nodeId) {
		Criteria criteria = factory.getCurrentSession().createCriteria(
				ModuleFunction.class);
		criteria.add(Restrictions.eq("moduleFunction.id", nodeId));
		criteria.addOrder(Order.asc("description"));

		List<ModuleFunction> moduleFunctions = criteria.list();
		List<TreeNode> treeNodes = new ArrayList<TreeNode>();

		if (moduleFunctions.size() > 0) {
			for (ModuleFunction function : moduleFunctions) {
				TreeNode node = new TreeNode(function.getId(),
						function.getDescription(), false);
				treeNodes.add(node);
			}
		} else {
			ModuleFunction function = (ModuleFunction) factory
					.getCurrentSession().get(ModuleFunction.class, nodeId);
			HttpServletRequest request = ServletActionContext.getRequest();

			String sActionFlag = "", sUrlAction = "";
			/*
			 * check the record in descriptor table if the action_flag = 1 means
			 * url_action else the node is url_descriptor value
			 */
			sActionFlag = String.valueOf(function.getModuleDescriptor()
					.getTypeFlag());

			if (sActionFlag.equals("1")) {
				sUrlAction = function.getModuleDescriptor().getUrlAction()
						+ "&";
			} else if (sActionFlag.equals("2")) {
				sUrlAction = /* request.getContextPath() */"#" + "/module/"
						+ function.getModuleDescriptor().getName() + "/";
			} else if (sActionFlag.equals("3")) {
				sUrlAction = /* request.getContextPath() */"#"
						+ "/module/window/genform?window.id="
						+ function.getModuleDescriptor().getWindow().getId()
						+ "&";
			} else {
				sUrlAction = /* request.getContextPath() */"#" + "/descriptor/"
						+ function.getModuleDescriptor().getUrlDescriptor()
						+ "?";
			}

			TreeNode node = new TreeNode(function.getId() + "create", "New",
					true);

			if (sActionFlag.equals("2")) {
				node.setHref(sUrlAction + "create");
				// node.setHrefTarget("main");
			} else {
				node.setHref(sUrlAction + "action=new");
				// node.setHrefTarget("main");
			}
			treeNodes.add(node);

			node = new TreeNode(function.getId() + "filter", "Search", true);

			if (sActionFlag.equals("2")) {
				node.setHref(sUrlAction + "filter");
				// node.setHrefTarget("main");
			} else {
				node.setHref(sUrlAction + "action=search");
				// node.setHrefTarget("main");
			}
			treeNodes.add(node);
		}

		return treeNodes;
	}
}
